    <!-- ***************************************************************************************************************
    *
    * Emits rows with the module description, recursively navigating through sub modules.
    *
    **************************************************************************************************************** -->
    <xsl:template name="emit-modules-description">
        <xsl:param name="baseUrl"/>
        <xsl:param name="indent" select="'0'"/>
        
        <xsl:variable name="pomUrl">
            <xsl:value-of select="$baseUrl"/>
            <xsl:value-of select="'/pom.xml'"/>
        </xsl:variable>
        
        <xsl:for-each select="document($pomUrl)/pom:project">                       
            <xsl:text>&#10;</xsl:text>
            <xsl:element name="tr">
                <xsl:element name="td">
                    <xsl:attribute name="class">
                        <xsl:value-of select="'indent-'"/>
                        <xsl:value-of select="$indent"/>
                    </xsl:attribute>
                    <xsl:value-of select="pom:name"/>
                </xsl:element>
                <xsl:element name="td">
                    <xsl:value-of select="pom:artifactId"/>
                </xsl:element>
                <xsl:element name="td">
                </xsl:element>
                <xsl:element name="td">
                    <xsl:value-of select="pom:description"/>
                </xsl:element>
            </xsl:element>
        </xsl:for-each>
        
        <xsl:for-each select="document($pomUrl)/pom:project/pom:modules/pom:module">                
                <xsl:call-template name="emit-modules-description">
                    <xsl:with-param name="baseUrl">
                        <xsl:value-of select="$baseUrl"/>
                        <xsl:value-of select="'/'"/>
                        <xsl:value-of select="."/>
                    </xsl:with-param>
                    <xsl:with-param name="indent">
                        <xsl:value-of select="($indent + 1)"/>
                    </xsl:with-param>
                </xsl:call-template>
        </xsl:for-each>
    </xsl:template>
    
    
    <!-- ***************************************************************************************************************
    *
    * Returns the latest version of an artifact.
    *
    **************************************************************************************************************** -->
    <xsl:template name="get-latest-version">
        <xsl:param name="groupId"/>
        <xsl:param name="artifactId"/>
        
        <xsl:variable name="queryUrl">
            <xsl:call-template name="get-mavenrepo-query-url">
                <xsl:with-param name="groupId" select="span[@class='nwXsltMacro_MavenRepoLatestVersion_groupId']"/>
                <xsl:with-param name="artifactId" select="span[@class='nwXsltMacro_MavenRepoLatestVersion_artifactId']"/>
            </xsl:call-template>
        </xsl:variable>
        <xsl:call-template name="find-version">
            <xsl:with-param name="response" select="document($queryUrl)"/>
        </xsl:call-template>
    </xsl:template>
    
    <!-- ***************************************************************************************************************
    *
    * Returns the issue management URL of an artifact.
    *
    **************************************************************************************************************** -->
    <xsl:template name="get-issue-management-url">
        <xsl:param name="groupId"/>
        <xsl:param name="artifactId"/>
        
        <xsl:variable name="pomUrl">
            <xsl:call-template name="get-mavenrepo-artifact-download-url">
                <xsl:with-param name="groupId" select="$groupId"/>
                <xsl:with-param name="artifactId" select="$artifactId"/>
                <xsl:with-param name="type" select="'pom'"/>
            </xsl:call-template>
        </xsl:variable>
        <xsl:for-each select="document($pomUrl)/pom:project">                
            <xsl:value-of select="'http://services.tidalwave.it/jira/'"/>
            <xsl:value-of select="pom:properties/pom:tidalwave.issues"/>
        </xsl:for-each>
    </xsl:template>
    
    <!-- ***************************************************************************************************************
    *
    * Returns the latest version of an artifact given the response.
    *
    **************************************************************************************************************** -->
    <xsl:template name="find-version">
        <xsl:param name="response"/>
        
        <xsl:value-of select="$response/response/result[@name='response']/doc/str[@name='latestVersion']" />
        <xsl:value-of select="$response/response/result[@name='response']/doc/str[@name='v']" />
    </xsl:template>
    
    <!-- ***************************************************************************************************************
    *
    * Returns the query url for an artifact.
    *
    **************************************************************************************************************** -->
    <xsl:template name="get-mavenrepo-query-url">
        <xsl:param name="groupId"/>
        <xsl:param name="artifactId"/>
        <xsl:param name="classifier"/>
        <xsl:param name="type"/>
        
        <xsl:value-of select="'http://search.maven.org/solrsearch/select?q=g:%22'"/>
        <xsl:value-of select="$groupId"/>
        <xsl:value-of select="'%22+AND+a:%22'"/>
        <xsl:value-of select="$artifactId"/>
        <xsl:if test="$classifier and ($classifier != 'bin')">
            <xsl:value-of select="'%22+AND+l:%22'"/>
            <xsl:value-of select="$classifier"/>
            <xsl:if test="$type">
                <xsl:value-of select="'%22+AND+p:%22'"/>
                <xsl:value-of select="$type"/>
            </xsl:if>
        </xsl:if>
        <xsl:value-of select="'%22&amp;rows=1&amp;wt=xml'"/>
    </xsl:template>

    <!-- ***************************************************************************************************************
    *
    * Returns the download URL for an artifact.
    *
    **************************************************************************************************************** -->
    <xsl:template name="get-mavenrepo-artifact-download-url">
        <xsl:param name="groupId"/>
        <xsl:param name="artifactId"/>
        <xsl:param name="classifier"/>
        <xsl:param name="type"/>
        
        <xsl:variable name="queryUrl">
            <xsl:call-template name="get-mavenrepo-query-url">
                <xsl:with-param name="groupId" select="$groupId"/>
                <xsl:with-param name="artifactId" select="$artifactId"/>
                <xsl:with-param name="type">
                    <xsl:choose>
                        <xsl:when test="$classifier and ($classifier = 'executable')">
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:value-of select="$type"/>
                        </xsl:otherwise>
                    </xsl:choose>
                </xsl:with-param>
                <xsl:with-param name="classifier" select="$classifier"/>
            </xsl:call-template>
        </xsl:variable>
        
        <xsl:for-each select="document($queryUrl)/response/result[@name='response']/doc">
            <xsl:variable name="xgroupId" select="str[@name='g']"/>
            <xsl:variable name="xartifactId" select="str[@name='a']"/>
            <xsl:variable name="xversion">
                <xsl:if test="str[@name='latestVersion']">
                    <xsl:value-of select="str[@name='latestVersion']"/>
                </xsl:if>
                <xsl:value-of select="str[@name='v']"/>
            </xsl:variable>
            <xsl:variable name="xtype">
                <xsl:choose>
                    <xsl:when test="$classifier and (($classifier = 'bin') or ($classifier = 'executable'))">
                        <xsl:value-of select="$type"/>
                    </xsl:when>
                    <xsl:when test="$type = 'pom'">
                        <xsl:value-of select="$type"/>
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:value-of select="str[@name='p']"/>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:variable>
        
            <xsl:value-of select="'http://search.maven.org/remotecontent?filepath='"/>
            <xsl:value-of select="translate($xgroupId, '.', '/')"/>
            <xsl:value-of select="'/'"/>
            <xsl:value-of select="$xartifactId"/>
            <xsl:value-of select="'/'"/>
            <xsl:value-of select="$xversion"/>
            <xsl:value-of select="'/'"/>
            <xsl:value-of select="$xartifactId"/>
            <xsl:value-of select="'-'"/>
            <xsl:value-of select="$xversion"/>
            <xsl:if test="$classifier">
                <xsl:value-of select="'-'"/>
                <xsl:value-of select="$classifier"/>
            </xsl:if>
            <xsl:value-of select="'.'"/>
            <xsl:value-of select="$xtype"/>
        </xsl:for-each>
    </xsl:template>

    <!-- ***************************************************************************************************************
    *
    * Returns the download URL for the source tarball from the SCM.
    * FIXME: only works with BitBucket
    * FIXME: it relies on Tidalwave property conventions, should instead compute the effective POM and query <pom:scm>
    *
    **************************************************************************************************************** -->
    <xsl:template name="get-scm-source-tarball-url">
        <xsl:param name="pomUrl"/>
        
        <xsl:for-each select="document($pomUrl)/pom:project">                
            <xsl:value-of select="pom:properties/pom:scm.repo.readonly.url"/>
            <xsl:value-of select="'/get/'"/>
            <xsl:value-of select="pom:version"/>
            <xsl:value-of select="'.tar.gz'"/>
        </xsl:for-each>
    </xsl:template>
    
    <!-- ***************************************************************************************************************
    *
    * Returns the browse URL for the sources from the SCM.
    * FIXME: only works with BitBucket
    * FIXME: it relies on Tidalwave property conventions, should instead compute the effective POM and query <pom:scm>
    *
    **************************************************************************************************************** -->
    <xsl:template name="get-scm-source-browse-url">
        <xsl:param name="pomUrl"/>
        
        <xsl:for-each select="document($pomUrl)/pom:project">                
            <xsl:value-of select="pom:properties/pom:scm.repo.readonly.url"/>
            <xsl:value-of select="'/src/'"/>
            <xsl:value-of select="pom:version"/>
        </xsl:for-each>
    </xsl:template>

    <!-- ***************************************************************************************************************
    *
    * Returns the raw URL for the sources from the SCM.
    * FIXME: only works with BitBucket
    * FIXME: it relies on Tidalwave property conventions, should instead compute the effective POM and query <pom:scm>
    *
    **************************************************************************************************************** -->
    <xsl:template name="get-scm-source-raw-base-url">
        <xsl:param name="pomUrl"/>
        
        <xsl:for-each select="document($pomUrl)/pom:project">                
            <xsl:value-of select="pom:properties/pom:scm.repo.readonly.url"/>
            <xsl:value-of select="'/raw/'"/>
            <xsl:value-of select="pom:version"/>
        </xsl:for-each>
    </xsl:template>
    
